@@ -2,7 +2,7 @@
 
    Project: FTP
 
-   Copyright (C) 2005-2013 Riccardo Mottola
+   Copyright (C) 2005-2016 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -36,44 +36,53 @@
 
 @interface AppController : NSObject
 {
-    IBOutlet NSWindow            *mainWin;
-    IBOutlet NSTableView         *localView;
-    IBOutlet NSTableView         *remoteView;
-    IBOutlet NSPopUpButton       *localPath;
-    IBOutlet NSPopUpButton       *remotePath;
-    IBOutlet NSButton            *buttDownload;
-    IBOutlet NSButton            *buttUpload;
-    IBOutlet NSTextField         *infoMessage;
-    IBOutlet NSTextField         *infoSpeed;
-    IBOutlet NSTextField         *infoSize;
-    IBOutlet NSProgressIndicator *progBar;
-    
-    IBOutlet NSWindow     *logWin;
-    IBOutlet NSTextView   *logTextField;
-    
-    IBOutlet NSPanel      *connectPanel;
-    IBOutlet NSTextField  *connAddress;
-    IBOutlet NSTextField  *connPort;
-    IBOutlet NSTextField  *connUser;
-    IBOutlet NSTextField  *connPass;
-    IBOutlet NSButton     *connAnon;
+  IBOutlet NSMenu *mainMenu;
 
-    IBOutlet NSPanel     *prefPanel;
-    IBOutlet NSMatrix    *portType;
+  IBOutlet NSWindow            *mainWin;
+  IBOutlet NSTableView         *localView;
+  IBOutlet NSTableView         *remoteView;
+  IBOutlet NSPopUpButton       *localPath;
+  IBOutlet NSPopUpButton       *remotePath;
+  IBOutlet NSButton            *buttDownload;
+  IBOutlet NSButton            *buttUpload;
+  IBOutlet NSTextField         *infoMessage;
+  IBOutlet NSTextField         *infoSpeed;
+  IBOutlet NSTextField         *infoSize;
+  IBOutlet NSProgressIndicator *progBar;
     
-    NSMutableDictionary  *textAttributes;
+  IBOutlet NSWindow     *logWin;
+  IBOutlet NSTextView   *logTextField;
     
-    FileTable   *localTableData;
-    FileTable   *remoteTableData;
-    FtpClient   *ftp;
-    LocalClient *local;
+  IBOutlet NSPanel      *connectPanel;
+  IBOutlet NSBox        *connServerBox;
+  IBOutlet NSBox        *connAccountBox;
+  IBOutlet NSTextField  *connAddress;
+  IBOutlet NSTextField  *connPort;
+  IBOutlet NSTextField  *connUser;
+  IBOutlet NSTextField  *connPass;
+  IBOutlet NSButton     *connAnon;
+  IBOutlet NSButton     *connCancelButt;
+  IBOutlet NSButton     *connConnectButt;
 
-    @private connectionModes    connMode;
-    @private struct timeval     beginTimeVal;
-    @private unsigned long long transferSize;
-    @private BOOL               threadRunning;
+  IBOutlet NSPanel     *prefPanel;
+  IBOutlet NSMatrix    *portType;
+  IBOutlet NSButton    *prefCancelButt;
+  IBOutlet NSButton    *prefSaveButt;
+    
+  NSMutableDictionary  *textAttributes;
+    
+  FileTable   *localTableData;
+  FileTable   *remoteTableData;
+  FtpClient   *ftp;
+  LocalClient *local;
 
-    @private NSConnection   *doConnection;
+  NSMutableArray *filesInProcess;
+  @private connectionModes    connMode;
+  @private NSTimeInterval     beginTimeVal;
+  @private unsigned long long transferSize;
+  @private BOOL               threadRunning;
+
+  @private NSConnection   *doConnection;
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotif;
@@ -101,9 +110,9 @@
 
 - (void)setThreadRunningState:(BOOL)flag;
 
-- (void)setTransferBegin:(NSString *)name :(unsigned long long)size;
-- (void)setTransferProgress:(NSNumber *)bytesTransferred;
-- (void)setTransferEnd:(NSNumber *)bytesTransferred;
+- (oneway void)setTransferBegin:(in bycopy NSString *)name :(unsigned long long)size;
+- (oneway void)setTransferProgress:(in bycopy NSNumber *)bytesTransferred;
+- (oneway void)setTransferEnd:(in bycopy NSNumber *)bytesTransferred;
 
 /** closes the open connections and quits the session with the remote server */
 - (IBAction)disconnect:(id)sender;
@@ -116,7 +125,7 @@
 
 /** Called by the server object to register itself */
 - (void)setServer:(id)anObject;
-- (void)appendTextToLog:(NSString *)textChunk;
+- (oneway void)appendTextToLog:(NSString *)textChunk;
 
 - (IBAction)showConnPanel:(id)sender;
 - (IBAction)connectConn:(id)sender;
@@ -132,8 +141,10 @@
 - (void)performRetrieveFile;
 - (void)performStoreFile;
 
-- (void)retrieveFiles:(NSArray *)files;
-- (void)storeFiles:(NSArray *)files;
+- (void)retrieveFiles;
+- (oneway void)fileRetrieved:(BOOL)success;
+- (void)storeFiles;
+- (oneway void)fileStored:(BOOL)success;
 
 @end
 
