@@ -1,7 +1,7 @@
 /* 
    Project: FTP
 
-   Copyright (C) 2005-2015 Riccardo Mottola
+   Copyright (C) 2005-2016 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -63,6 +63,53 @@
 
 - (void)awakeFromNib
 {
+  NSMenu *menu;
+  NSMenuItem *mi;
+  
+  /* connection panel */
+  [connServerBox setTitle:NSLocalizedString(@"Server Address and Port", @"Server Address and Port")];
+  [connAccountBox setTitle:NSLocalizedString(@"Account", @"Account")];
+  [connectPanel setTitle:NSLocalizedString(@"Connect", @"Connect")];
+  [connAnon setTitle:NSLocalizedString(@"Anonymous", @"Anonymous connection")];
+  [connCancelButt setTitle:NSLocalizedString(@"Cancel", @"Cancel")];
+  [connConnectButt setTitle:NSLocalizedString(@"Connect (action)", @"Connect (action)")];
+  
+  /* main window */
+  [[localPath itemAtIndex:0] setTitle:NSLocalizedString(@"local view", @"local view")];
+  [[remotePath itemAtIndex:0] setTitle:NSLocalizedString(@"remote view", @"remote view")];
+  [[[localView tableColumnWithIdentifier:@"filename"] headerCell] setStringValue:NSLocalizedString(@"Name", @"filename table")];
+  [[[remoteView tableColumnWithIdentifier:@"filename"] headerCell] setStringValue:NSLocalizedString(@"Name", @"filename table")];
+  
+  /* menus */
+  mi = [mainMenu itemWithTitle:@"Local"];
+  menu = [mi submenu];
+  [menu setTitle:NSLocalizedString(@"Local", @"Local")];
+  mi = [menu itemWithTag:1];
+  [mi setTitle:NSLocalizedString(@"Rename...", @"Rename...")];
+  mi = [menu itemWithTag:2];
+  [mi setTitle:NSLocalizedString(@"New Folder...", @"New Folder....")];
+  mi = [menu itemWithTag:3];
+  [mi setTitle:NSLocalizedString(@"Delete", @"Delete")];
+  mi = [menu itemWithTag:4];
+  [mi setTitle:NSLocalizedString(@"Refresh", @"Refresh")];
+
+  mi = [mainMenu itemWithTitle:@"Remote"];
+  menu = [mi submenu];
+  [menu setTitle:NSLocalizedString(@"Remote", @"Remote")];
+  mi = [menu itemWithTag:1];
+  [mi setTitle:NSLocalizedString(@"Rename...", @"Rename...")];
+  mi = [menu itemWithTag:2];
+  [mi setTitle:NSLocalizedString(@"New Folder...", @"New Folder....")];
+  mi = [menu itemWithTag:3];
+  [mi setTitle:NSLocalizedString(@"Delete", @"Delete")];
+  mi = [menu itemWithTag:4];
+  [mi setTitle:NSLocalizedString(@"Refresh", @"Refresh")];
+
+  /* log */
+  [logWin setTitle:NSLocalizedString(@"Connection Log", @"Connection Log")];
+
+  [logTextField setSelectable:YES];
+  [logTextField setEditable:NO];
 }
 
 - (void)applicationDidFinishLaunching:(NSNotification *)aNotif
@@ -78,12 +125,12 @@
     defaults = [NSUserDefaults standardUserDefaults];
     readValue = [defaults stringForKey:connectionModeKey];
 
-    /* if no value was set for the key we set port as mode */
+    /* if no value was set for the key we set passive as mode */
     if ([readValue isEqualToString:@"default"])
         connMode = defaultMode;
-    else if ([readValue isEqualToString:@"port"]  || readValue == nil)
+    else if ([readValue isEqualToString:@"port"] )
         connMode = portMode;
-    else if ([readValue isEqualToString:@"passive"])
+    else if ([readValue isEqualToString:@"passive"] || readValue == nil)
         connMode = passiveMode;
     else
         NSLog(@"Unrecognized value in user preferences for %@: %@", connectionModeKey, readValue);
@@ -335,7 +382,7 @@
   else if (sender == remoteTableData)
     {
       NSLog(@"will upload: %@", arr);
-      [self storeFiles:arr];
+      [self storeFiles];
     }
   [arr release];
 }
@@ -373,92 +420,137 @@
 /** retrieves using selection, by constructing an array and calling retrieveFiles */
 - (void)performRetrieveFile
 {
-  NSMutableArray *selArray;
   NSEnumerator   *elemEnum;
   FileElement    *fileEl;
   id             currEl;
 
   /* make a copy of the selection */
-  selArray = [[NSMutableArray alloc] init];
+  [filesInProcess release];
+  filesInProcess = [[NSMutableArray alloc] init];
   elemEnum = [remoteView selectedRowEnumerator];
 
   while ((currEl = [elemEnum nextObject]) != nil)
     {
       fileEl = [remoteTableData elementAtIndex:[currEl intValue]];
-      [selArray addObject:fileEl];
+      [filesInProcess addObject:fileEl];
     }
     
-  [self retrieveFiles:selArray];
-  [selArray release];
+  [self retrieveFiles];
 }
 
 /** stores using selection, by constructing an array and calling storeFiles */
 - (void)performStoreFile
 {
-  NSMutableArray *selArray;
   NSEnumerator   *elemEnum;
   FileElement    *fileEl;
   id             currEl;
   
   /* make a copy of the selection */
-  selArray = [[NSMutableArray alloc] init];
+  [filesInProcess release];
+  filesInProcess = [[NSMutableArray alloc] init];
   elemEnum = [localView selectedRowEnumerator];
     
   while ((currEl = [elemEnum nextObject]) != nil)
     {
       fileEl = [localTableData elementAtIndex:[currEl intValue]];
-      [selArray addObject:fileEl];
+      [filesInProcess addObject:fileEl];
     }
-  [self storeFiles:selArray];
-  [selArray release];
+  [self storeFiles];
 }
 
 /** Retrieves Array of FileElements */
-- (void)retrieveFiles:(NSArray *)files
+- (void)retrieveFiles
 {
-  NSUInteger i;
-
-  [self setThreadRunningState:YES];
-  for (i = 0; i < [files count]; i++)
+  if([filesInProcess count] > 0)
     {
       FileElement *fEl;
 
-      fEl = [files objectAtIndex:i];
-      NSLog(@"should download (performStore): %@", [fEl name]);
-      if ([ftp retrieveFile:fEl to:local beingAt:0])
+      [self setThreadRunningState:YES];
+      fEl = [filesInProcess objectAtIndex:0];
+      NSLog(@"should download (performRETRIEVE): %@", [fEl name]);
+      [ftp retrieveFile:fEl to:local];
+    }
+}
+
+/* called by the worker thread when the element got processed */
+- (oneway void)fileRetrieved:(BOOL)success
+{
+  FileElement *fEl;
+
+  fEl = [filesInProcess objectAtIndex:0];
+  if (success)
+    {
+      if (![localTableData containsFileName:[fEl name]])
         {
-          if (![localTableData containsFileName:[fEl name]])
-            [localTableData addObject:fEl];
+          FileElement *fEl2;
+              
+          fEl2 = [[FileElement alloc] initWithPath:[[local workingDir] stringByAppendingPathComponent:[fEl name]] andAttributes:[fEl attributes]];
+          [localTableData addObject:fEl2];
+          [fEl2 release];
         }
     }
-  [localView deselectAll:self];
-  [localView reloadData];
-  [self setThreadRunningState:NO];
+  [filesInProcess removeObjectAtIndex:0];
+  if ([filesInProcess count] > 0)
+    {
+      [self retrieveFiles];
+    }
+  else
+    {
+      [localView deselectAll:self];
+      [localView reloadData];
+      [self setThreadRunningState:NO];
+      [filesInProcess release];
+      filesInProcess = nil;
+    }
 }
 
 /** Stores Array of FileElements */
-- (void)storeFiles:(NSArray *)files
+- (void)storeFiles
 {
-  NSUInteger i;
-
-  [self setThreadRunningState:YES];
-  for (i = 0; i < [files count]; i++)
+  if([filesInProcess count] > 0)
     {
       FileElement *fEl;
 
-      fEl = [files objectAtIndex:i];
-      NSLog(@"should upload (performStore): %@", [fEl name]);
-      if ([ftp storeFile:fEl from:local beingAt:0])
+      [self setThreadRunningState:YES];
+      fEl = [filesInProcess objectAtIndex:0];
+      NSLog(@"should download (performStore): %@", [fEl name]);
+      [ftp storeFile:fEl from:local];
+    }
+}
+
+/* called by the worker thread when the element got processed */
+- (oneway void)fileStored:(BOOL)success
+{
+  FileElement *fEl;
+
+  fEl = [filesInProcess objectAtIndex:0];
+  if (success)
+    {
+      if (![remoteTableData containsFileName:[fEl name]])
         {
-          if (![remoteTableData containsFileName:[fEl name]])
-            [remoteTableData addObject:fEl];
+          FileElement *fEl2;
+              
+          fEl2 = [[FileElement alloc] initWithPath:[[ftp workingDir] stringByAppendingPathComponent:[fEl name]] andAttributes:[fEl attributes]];
+          [remoteTableData addObject:fEl2];
+          [fEl2 release];
         }
     }
-  [remoteView deselectAll:self];
-  [remoteView reloadData];
-  [self setThreadRunningState:NO];
+  [filesInProcess removeObjectAtIndex:0];
+  if ([filesInProcess count] > 0)
+    {
+      [self storeFiles];
+    }
+  else
+    {
+      [remoteView deselectAll:self];
+      [remoteView reloadData];
+      [self setThreadRunningState:NO];
+      [filesInProcess release];
+      filesInProcess = nil;
+    }
 }
 
+
 - (IBAction)downloadButton:(id)sender
 {
   if (threadRunning)
@@ -502,8 +594,8 @@
   for (i = 0; i < [selArray count]; i++)
     {
       fileEl = [selArray objectAtIndex:i];
-      [local deleteFile:fileEl beingAt:0];
-      [localTableData removeObject:fileEl];
+      if([local deleteFile:fileEl beingAt:0])
+	[localTableData removeObject:fileEl];
     }
   [localView deselectAll:self];
   [localView reloadData];
@@ -531,8 +623,8 @@
   for (i = 0; i < [selArray count]; i++)
     {
       fileEl = [selArray objectAtIndex:i];
-      [ftp deleteFile:fileEl beingAt:0];
-      [remoteTableData removeObject:fileEl];
+      if ([ftp deleteFile:fileEl beingAt:0])
+	[remoteTableData removeObject:fileEl];
     }
   [remoteView deselectAll:self];
   [remoteView reloadData];
@@ -685,17 +777,11 @@
   [self readDirWith:ftp toTable:remoteTableData andView:remoteView];
 }
 
-- (void)setTransferBegin:(NSString *)name :(unsigned long long)size
+- (oneway void)setTransferBegin:(in bycopy NSString *)name :(unsigned long long)size
 {
     [infoMessage setStringValue:name];
     [progBar setDoubleValue:0];
-#ifdef WIN32
-    DWORD msecs = timeGetTime();
-    beginTimeVal.tv_sec=msecs/1000;
-    beginTimeVal.tv_usec=(msecs - beginTimeVal.tv_sec*1000) * 1000; 
-#else
-    gettimeofday(&beginTimeVal, NULL);
-#endif
+    beginTimeVal = [NSDate timeIntervalSinceReferenceDate];
     transferSize = size;
     NSLog(@"begin transfer size: %llu", transferSize);
     if (transferSize == 0)
@@ -706,9 +792,9 @@
     [mainWin displayIfNeeded];
 }
 
-- (void)setTransferProgress:(NSNumber *)bytesTransferred
+- (oneway void)setTransferProgress:(in bycopy NSNumber *)bytesTransferred
 {
-  struct timeval currTimeVal;
+  NSTimeInterval currTimeVal;
   float    speed;
   NSString *speedStr;
   NSString *sizeStr;
@@ -716,14 +802,8 @@
   unsigned long long bytes;
 
   bytes = [bytesTransferred unsignedLongLongValue];
-#ifdef WIN32
-    DWORD msecs = timeGetTime();
-    currTimeVal.tv_sec=msecs/1000;
-    currTimeVal.tv_usec=(msecs - currTimeVal.tv_sec*1000) * 1000; 
-#else
-    gettimeofday(&currTimeVal, NULL);
-#endif
-    speed = (float)((double)bytes / (double)(currTimeVal.tv_sec - beginTimeVal.tv_sec));
+  currTimeVal = [NSDate timeIntervalSinceReferenceDate];
+  speed = (float)((double)bytes / (double)(currTimeVal - beginTimeVal));
 
     if (transferSize > 0)
       {
@@ -751,14 +831,12 @@
         sizeStr = [sizeStr initWithFormat:@"%3.2f : %3.2f MB", (double)bytes/(1024*1024), (double)transferSize/(1024*1024)];
     [infoSize setStringValue:sizeStr];
     [sizeStr release];
-
-    [[NSRunLoop currentRunLoop] runUntilDate:[NSDate distantPast]];
 }
 
-- (void)setTransferEnd:(NSNumber *)bytesTransferred
+- (oneway void)setTransferEnd:(in bycopy NSNumber *)bytesTransferred
 {
-  struct timeval currTimeVal;
-  double         deltaT;
+  NSTimeInterval currTimeVal;
+  NSTimeInterval deltaT;
   float          speed;
   NSString       *speedStr;
   NSString       *sizeStr;
@@ -766,15 +844,8 @@
   unsigned long long bytes;
 	
   bytes = [bytesTransferred unsignedLongLongValue];
-
-#ifdef WIN32
-    DWORD msecs = timeGetTime();
-    currTimeVal.tv_sec=msecs/1000;
-    currTimeVal.tv_usec=(msecs - currTimeVal.tv_sec*1000) * 1000; 
-#else
-    gettimeofday(&currTimeVal, NULL);
-#endif
-    deltaT = (currTimeVal.tv_sec - beginTimeVal.tv_sec)+((double)(currTimeVal.tv_usec - beginTimeVal.tv_usec)/1000000);
+  currTimeVal = [NSDate timeIntervalSinceReferenceDate];
+  deltaT = (currTimeVal - beginTimeVal);
     speed = (float)((double)bytes / deltaT);
     NSLog(@"Elapsed time: %f", (float)deltaT);
     percent = ((double)bytes / (double)transferSize) * 100;
@@ -891,7 +962,7 @@
   ftp = (FtpClient*)[anObject retain];
 }
 
-- (void)appendTextToLog:(NSString *)textChunk
+- (oneway void)appendTextToLog:(NSString *)textChunk
 {
   NSAttributedString *attrStr;
     
