@@ -2,7 +2,7 @@
 
  Project: FTP
 
- Copyright (C) 2005-2013 Free Software Foundation
+ Copyright (C) 2005-2016 Free Software Foundation
 
  Author: Riccardo Mottola
 
@@ -47,22 +47,22 @@
 - (id)initWithController:(id)cont;
 
 /** returns the current working directory */
-- (NSString *)workingDir;
+- (bycopy NSString *)workingDir;
 
 - (void)setWorkingDirWithCString:(char *)dir;
 - (void)setWorkingDir:(NSString *)dir;
 - (void)changeWorkingDir:(NSString *)dir;
 - (BOOL)createNewDir:(NSString *)dir;
-- (void)deleteFile:(FileElement *)file beingAt:(int)depth;
+- (BOOL)deleteFile:(FileElement *)file beingAt:(int)depth;
 - (BOOL)renameFile:(FileElement *)file to:(NSString *)name;
 
-- (NSArray *)workDirSplit;
+- (bycopy NSArray *)workDirSplit;
 
 /** returns an array with the directory listing */
-- (NSArray *)dirContents;
+- (bycopy NSArray *)dirContents;
 
 /** returns the current home directory */
-- (NSString *)homeDir;
+- (bycopy NSString *)homeDir;
 
 @end
 
