@@ -1,7 +1,7 @@
 /*
  Project: FTP
 
- Copyright (C) 2005-2015 Riccardo Mottola
+ Copyright (C) 2005-2017 Riccardo Mottola
 
  Author: Riccardo Mottola
 
@@ -55,7 +55,7 @@
 #define MAX_CONTROL_BUFF 2048
 #define MAX_DATA_BUFF 2048
 
-#if defined(__linux__) || defined (__BSD_VISIBLE)
+#if defined(__linux__) || defined (__BSD_VISIBLE) || defined (NetBSD) || defined (__APPLE__)
 #define socklentype socklen_t
 #else
 #define socklentype int
@@ -395,6 +395,15 @@ int getChar(streamStruct* ss)
   return retVal;
 }
 
+- (oneway void)retrieveFile:(FileElement *)file to:(LocalClient *)localClient
+{
+  BOOL gotFile;
+  
+  gotFile = [self retrieveFile:file to:localClient beingAt:0];
+  
+  [controller fileRetrieved:gotFile];
+}
+
 - (BOOL)retrieveFile:(FileElement *)file to:(LocalClient *)localClient beingAt:(int)depth;
 {
   NSString           *fileName;
@@ -474,8 +483,16 @@ int getChar(streamStruct* ss)
 
     if(replyCode != 150)
       {
-        [controller showAlertDialog:@"Unexpected server error."];
-        NSLog(@"Unexpected condition in retrieve");
+	if (replyCode >= 400)
+	  {
+	    [controller showAlertDialog:[reply objectAtIndex:0]];
+	    [self logIt: [reply objectAtIndex:0]];
+	  }
+	else
+	  {
+	    [controller showAlertDialog:@"Unexpected server error."];
+	    NSLog(@"Unexpected condition in retrieve");
+	  }
         return NO; /* we have an error or some unexpected condition */
       }
     else
@@ -521,7 +538,7 @@ int getChar(streamStruct* ss)
         return NO;
     }
     
-    localFileStream = fopen([localPath cString], "w");
+    localFileStream = fopen([localPath cString], "wb");
     if (localFileStream == NULL)
     {
         [controller showAlertDialog:@"Opening of local file failed.\nCheck permissions and free space."];
@@ -532,7 +549,6 @@ int getChar(streamStruct* ss)
     totalBytes = 0;
     progressIncBytes = 0;
     gotFile = NO;
-    [controller setThreadRunningState:YES];
     [controller setTransferBegin:fileName :fileSize];
     while (!gotFile)
     {
@@ -565,8 +581,17 @@ int getChar(streamStruct* ss)
     [self closeDataStream];
     [self readReply:&reply];
     [reply release];
-    [controller setThreadRunningState:NO];
-  return gotFile;
+
+    return gotFile;
+}
+
+- (oneway void)storeFile:(FileElement *)file from:(LocalClient *)localClient
+{
+  BOOL gotFile;
+  
+  gotFile = [self storeFile:file from:localClient beingAt:0];
+  
+  [controller fileStored:gotFile];
 }
 
 - (BOOL)storeFile:(FileElement *)file from:(LocalClient *)localClient beingAt:(int)depth
@@ -649,7 +674,7 @@ int getChar(streamStruct* ss)
     replyCode = [self readReply:&reply];
     NSLog(@"%d reply is %@: ", replyCode, [reply objectAtIndex:0]);
 
-    if (replyCode >= 550 && replyCode <= 559)
+    if (replyCode >= 400 && replyCode <= 559)
     {
         [controller showAlertDialog:[reply objectAtIndex:0]];
         [self logIt: [reply objectAtIndex:0]];
@@ -666,10 +691,10 @@ int getChar(streamStruct* ss)
     }
 
 
-    localFileStream = fopen([localPath cString], "r");
+    localFileStream = fopen([localPath cString], "rb");
     if (localFileStream == NULL)
     {
-        [controller showAlertDialog:@"Opening of local file failed.\n Check permissions."];
+        [controller showAlertDialog:@"Opening of local file failed.\nCheck permissions."];
         perror("local fopen failed");
         return NO;
     }
@@ -677,7 +702,6 @@ int getChar(streamStruct* ss)
     totalBytes = 0;
     progressIncBytes = 0;
     gotFile = NO;
-    [controller setThreadRunningState:YES];
     [controller setTransferBegin:fileName :fileSize];
     while (!gotFile)
     {
@@ -691,9 +715,11 @@ int getChar(streamStruct* ss)
                 NSLog(@"feof");
         } else
         {
-            if (write(localSocket, buff, bytesRead) < bytesRead)
+          int sentBytes;
+          
+          if ((sentBytes = send(localSocket, buff, bytesRead, 0)) < bytesRead)
             {
-                NSLog(@"socket write error, store file");
+                NSLog(@"socket write error, store file. Wrote %d of %d", sentBytes, bytesRead);
             }
             totalBytes += bytesRead;
             progressIncBytes += bytesRead;
@@ -710,11 +736,10 @@ int getChar(streamStruct* ss)
     [self closeDataStream];
     [self readReply:&reply];
     [reply release];
-    [controller setThreadRunningState:NO];
   return gotFile;
 }
 
-- (void)deleteFile:(FileElement *)file beingAt:(int)depth
+- (BOOL)deleteFile:(FileElement *)file beingAt:(int)depth
 {
   NSString           *fileName;
   NSString           *command;
@@ -734,7 +759,7 @@ int getChar(streamStruct* ss)
         if (depth > 3)
         {
             NSLog(@"Max depth reached: %d", depth);
-            return;
+            return NO;
         }
 
         pristineRemotePath = [[self workingDir] retain];
@@ -761,8 +786,16 @@ int getChar(streamStruct* ss)
     [self writeLine:command];
     replyCode = [self readReply:&reply];
     NSLog(@"%d reply is %@: ", replyCode, [reply objectAtIndex:0]);
-    [reply release];
     
+    if(replyCode >= 400)
+      {
+	[controller showAlertDialog:[reply objectAtIndex:0]];
+        [self logIt: [reply objectAtIndex:0]];
+	[reply release];
+        return NO; /* we have an error or some unexpected condition */
+      }
+    [reply release];
+    return YES;
 }
 
 /* initialize a connection */
@@ -893,9 +926,7 @@ int getChar(streamStruct* ss)
 - (int)initDataConn
 {
     socklentype addrLen; /* socklen_t on some systems ? */
-    int         socketReuse;
-    
-    socketReuse = YES;
+    const int         socketReuse = 1;
 
     /* passive mode */
     if (usesPassive)
@@ -993,11 +1024,11 @@ int getChar(streamStruct* ss)
     /* linux is happier if we set both ends that way */
     if (usesPorts == NO)
     {
-        if (setsockopt(dataSocket, SOL_SOCKET, SO_REUSEADDR, &socketReuse, sizeof (socketReuse)) < 0)
+        if (setsockopt(dataSocket, SOL_SOCKET, SO_REUSEADDR, &socketReuse, (socklentype) sizeof (socketReuse)) < 0)
         {
             perror("ftpclient: setsockopt (reuse address) on data");
         }
-        if (setsockopt(controlSocket, SOL_SOCKET, SO_REUSEADDR, &socketReuse, sizeof (socketReuse)) < 0)
+        if (setsockopt(controlSocket, SOL_SOCKET, SO_REUSEADDR, &socketReuse, (socklentype) sizeof (socketReuse)) < 0)
         {
             perror("ftpclient: setsockopt (reuse address) on control");
         }
@@ -1046,8 +1077,10 @@ int getChar(streamStruct* ss)
         tempStr = [NSString stringWithFormat:@"PORT %u,%u,%u,%u,%u,%u", addr.ipv4[0], addr.ipv4[1], addr.ipv4[2], addr.ipv4[3], p1, p2];
         [self writeLine:tempStr];
         NSLog(@"port str: %@", tempStr);
-        if ((returnCode = [self readReply:&reply]) != 200)
+        returnCode = [self readReply:&reply];
+        if (returnCode != 200)
         {
+          NSLog(@"Error return code of PORT: %d", returnCode);
           if(reply && [reply count] > 0)
             {
               NSLog(@"error occoured in port command: %@", [reply objectAtIndex:0]);
@@ -1075,16 +1108,11 @@ int getChar(streamStruct* ss)
         if ((localSocket = accept(dataSocket, &from, &fromLen)) < 0)
         {
             perror("accepting socket, initDataStream: ");
+            return -1;
         }
         initStream(&dataStream, localSocket);
     }
-/*
-    if (dataStream == NULL)
-    {
-        perror("data stream opening failed");
-        return -1;
-    } */
-    NSLog(@"data stream open");
+
     return 0;
 }
 
@@ -1141,7 +1169,7 @@ int getChar(streamStruct* ss)
 
 
 /* RM again: a better path limit is needed */
-- (NSArray *)dirContents
+- (bycopy NSArray *)dirContents
 {
     int                ch;
     char               buff[MAX_DATA_BUFF];
