@@ -1,7 +1,7 @@
 /*
    Project: FTP
 
-   Copyright (C) 2005-2013 Riccardo Mottola
+   Copyright (C) 2005-2016 Riccardo Mottola
 
    Author: Riccardo Mottola
 
@@ -41,8 +41,7 @@
 
 /*
  creates a new directory
- tries to guess if the given dir is relative (no starting /) or absolute
- Is this portable to non-unix OS's?
+ If the path is absolute, use it directly, else append to wokding directory
  */
 - (BOOL)createNewDir:(NSString *)dir
 {
@@ -51,7 +50,7 @@
     BOOL          isDir;
 
     fm = [NSFileManager defaultManager];
-    if ([dir hasPrefix:@"/"])
+    if ([dir isAbsolutePath])
     {
         NSLog(@"%@ is an absolute path", dir);
         localPath = dir;
@@ -69,7 +68,7 @@
         return YES;
 }
 
-- (NSArray *)dirContents
+- (bycopy NSArray *)dirContents
 {
     NSFileManager   *fm;
     NSArray         *fileNames;
@@ -100,14 +99,18 @@
     return [NSArray arrayWithArray:listArr];
 }
 
-- (void)deleteFile:(FileElement *)file beingAt:(int)depth
+- (BOOL)deleteFile:(FileElement *)file beingAt:(int)depth
 {
   NSFileManager      *fm;
 
   fm = [NSFileManager defaultManager];
   
   if ([fm removeFileAtPath:[file path] handler:nil] == NO)
-    NSLog(@"an error occoured during local delete");
+    {
+      NSLog(@"an error occoured during local delete");
+      return NO;
+    }
+  return YES;
 }
 
 
